function imfftdemo(action)

if ~ ischar(action)
  im = action;
  action = 'Initialize';
else
  h = get(gcbf,'UserData');
end

switch action
 case 'Initialize'
  % create the main figure
  h.figh = figure('name', 'Image FFT Demo', ...
		 'numbertitle','off','Position',[40 0 950 700],...
		 'units', 'pixels', 'doublebuffer', 'on');
  h.Axis1 = subplot(2,2,1);
  [h.width h.height] = size(im);
  h.im = double(im);
  h.Image1 = imagesc(im);
  h.SumImSq = sum(double(im(:)).^2);
  set(h.Axis1, 'XTick', [], 'YTick', []);
  colormap(gray(256));
  axis image;
  title('Original')
  
  h.Axis2 = subplot(2,2,2);
  h.Fim = fftshift(fft2(im));
  aim = abs(h.Fim);
  h.sFim = -sort(-aim(:));
  h.Fnorm = mean(im(:)) / h.Fim(1+h.height/2, 1+h.width/2);
  h.Image2 = imagesc((1:h.width)-1-h.width/2, (1:h.height)-1-h.height/2, ...
		     log(abs(h.Fim)));
  axis image
  set(h.Image2, 'ButtonDownFcn', 'imfftdemo FTclick');
  title('Magnitude Spectrum')
  
  h.Axis3 = subplot(2,2,3);
  h.Mask = zeros(h.height, h.width);
  h.Image3 = image((1:h.width)-1-h.width/2, (1:h.height)-1-h.height/2, h.Mask);
  axis image
  set(h.Image3, 'ButtonDownFcn', 'imfftdemo FTclick');
  set(h.Image3, 'CDataMapping', 'scaled');
  set(h.Axis3, 'CLim', [0 1]);
  title('Spectrum Mask')
  
  h.Axis4 = subplot(2,2,4);
  h.Image4 = image(zeros(h.height, h.width));
  title('Reconstructed Image')
  axis image
  
  h.FFTValue = uicontrol(gcf, ...
			 'Style', 'text', ...
			 'Position', [830 550 100 20], ...
			 'String', 'Value in bin');
  uicontrol(gcf, ...
	    'Style', 'text', ...
	    'Position', [830 570 100 20], ...
	    'String', 'Value');
  
  h.MaskPercent = uicontrol(gcf, ...
			    'Style', 'text', ...
			    'Position', [830 500 100 20], ...
			    'String', 'On %');
  uicontrol(gcf, ...
	    'Style', 'text', ...
	    'Position', [830 520 100 20], ...
	    'String', '% On');
  
  h.Error = uicontrol(gcf, ...
		       'Style', 'text', ...
		       'Position', [830 450 100 20], ...
		       'String', 'Error %');
  uicontrol(gcf, ...
	    'Style', 'text', ...
	    'Position', [830 470 100 20], ...
	    'String', '% Error');
  
  h.PCntSlider = uicontrol(gcf, ...
			   'Style', 'slider', ...
			   'Position', [525 400 20 250], ...
			   'Min', 0, 'Max', 1, 'Value', 0, ...
			   'SliderStep', [1/length(im(:)) 10/length(im(:))], ...
			   'CallBack', 'imfftdemo Slider');
  
  h.SurfFig = figure('doublebuffer', 'on');
  h.Surf = surf(zeros(h.height, h.width));
  colormap(gray(256));
  axis([ 1 h.width 1 h.height -20 255 ]);
  xlabel('X')
  ylabel('Y')
  zlabel('I')
  
  figure(h.figh);
  
  set(gcf, 'UserData', h);
  
 case 'FTclick'
  hCurrAxis = get(gco,'parent');
  CurrPoint = get(hCurrAxis, 'currentpoint');
  SelType = get(gcf, 'SelectionType');
  [x,y] = deal(CurrPoint(1,1), CurrPoint(1,2));
  
  x = round(x);
  y = round(y);
  
  if strcmp(SelType, 'normal')
    nval = 1;
  else
    nval = 0;
  end
  
  h.Mask(y + 1 + h.height/2, x + 1 + h.width/2) = nval;
  if x > -h.width/2 & y > -h.height/2
    h.Mask(-y + 1 + h.height/2, -x + 1 + h.width/2) = nval;
  end
  
  val = h.Fim(y + 1 + h.height/2, x + 1 + h.width/2);
  set(h.FFTValue, 'String', num2str(val * h.Fnorm));
  localUpdate(h);
  
 case 'Slider'
  val = get(gco, 'Value');
  val = h.sFim(fix(val * (length(h.sFim) - 1)) + 1);
  h.Mask = abs(h.Fim) >= val;
  
  localUpdate(h);
  
end

function localUpdate(h)
  set(h.Image3, 'CData', h.Mask);
  set(h.MaskPercent, 'String', [num2str(100*sum(h.Mask(:))/length(h.Mask(:))) '%']);
  
  nim = real(ifft2(ifftshift(h.Fim .* h.Mask)));
  set(h.Image4, 'CData', nim);
  err = h.im - nim;
  set(h.Error, 'String', [num2str(100*sqrt(sum(err(:).^2)/h.SumImSq)) '%']);
  
  figure(h.SurfFig);
  set(h.Surf, 'ZData', nim);
  set(h.Surf, 'CData', nim);
  axis([ 1 h.width 1 h.height -20 255 ]);
  colormap(gray(256));
  figure(h.figh);

  set(gcf, 'UserData', h);
  
  
